# -*- coding: utf-8 -*-

"""
***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************
"""

from qgis.PyQt.QtCore import QCoreApplication
from qgis.core import (QgsProcessing,
                       QgsProcessingAlgorithm,
                       QgsProcessingParameterVectorLayer,
                       QgsProcessingParameterString,
                       QgsProcessingOutputVectorLayer)
from qgis import processing
from PyQt5.QtCore import QVariant
from qgis.core import QgsField, QgsFields
from qgis.core import QgsExpression
from qgis.core import QgsVectorLayer, QgsVectorDataProvider
from qgis.PyQt.QtCore import QVariant
from qgis.core import QgsProject, QgsWkbTypes

class AreaLengthProcessingAlgorithm(QgsProcessingAlgorithm):

    INPUT = 'INPUT'
    OUTPUT = 'OUTPUT'

    def tr(self, string):
        """
        Returns areas, perimeters and lengths of polygon and line vector layers.
        """
        return QCoreApplication.translate('Processing', string)

    def createInstance(self):
        return AreaLengthProcessingAlgorithm()

    def name(self):
        return 'calculate_area_length'

    def displayName(self):
        return self.tr('Calculate Areas and Length')

    def group(self):
        return self.tr('Calculations')

    def groupId(self):
        return 'calculations'

    def shortHelpString(self):
        return self.tr("Script destinado a calcular áreas e perímetros de polígonos, e comprimentos de linhas. Os cálculos são planimétricos, e só deve ser usado com layers vectoriais projectadas.")

    def initAlgorithm(self, config=None):
        """
        Input layer: any vector layer; Type: Polygon or Line.
        """
        self.addParameter(
            QgsProcessingParameterVectorLayer(
                self.INPUT,
                self.tr('Input layer'),
                [QgsProcessing.TypeVectorLine,QgsProcessing.TypeVectorPolygon]
            )
        )
        self.addOutput(
            QgsProcessingOutputVectorLayer(
                self.OUTPUT,
                self.tr('Output layer')
            )
        )

    def processAlgorithm(self, parameters, context, feedback):
        source = self.parameterAsVectorLayer(
            parameters,
            self.INPUT,
            context
        )
        
        # Calculate areas, perimeters and lengths
        features = source.getFeatures()
        for feature in features:
            geom = feature.geometry()
        if geom.type() == QgsWkbTypes.PolygonGeometry:

            field_index_list = []            
            field_name = "Area_m2"
            if source.fields().indexFromName(field_name)>0:
                field_index_list.append(source.fields().indexFromName(field_name))            
            field_name = "AREA_M2"
            if source.fields().indexFromName(field_name)>0:
                field_index_list.append(source.fields().indexFromName(field_name))            
            field_name = "area_m2"
            if source.fields().indexFromName(field_name)>0:    
                field_index_list.append(source.fields().indexFromName(field_name))

            source.dataProvider().deleteAttributes(field_index_list)
            source.updateFields()
            pr = source.dataProvider()
            attr = pr.addAttributes([QgsField('Area_m2', QVariant.Double, 'double', 20, 2)])
            source.updateFields()
            caps = pr.capabilities()
            field_index = source.fields().indexFromName('Area_m2')
            if caps & QgsVectorDataProvider.ChangeAttributeValues:
                for feature in source.getFeatures():
                    attr = {field_index : round(feature.geometry().area(), 2)}
                    source.dataProvider().changeAttributeValues({feature.id() : attr})
            source.updateFields()

            field_index_list = []            
            field_name = "Area_ha"
            if source.fields().indexFromName(field_name)>0:
                field_index_list.append(source.fields().indexFromName(field_name))            
            field_name = "AREA_HA"
            if source.fields().indexFromName(field_name)>0:
                field_index_list.append(source.fields().indexFromName(field_name))            
            field_name = "area_ha"
            if source.fields().indexFromName(field_name)>0:    
                field_index_list.append(source.fields().indexFromName(field_name))
            
            source.dataProvider().deleteAttributes(field_index_list)
            source.updateFields()
            pr = source.dataProvider()
            attr = pr.addAttributes([QgsField('Area_ha', QVariant.Double, 'double', 20, 2)])
            source.updateFields()
            caps = pr.capabilities()
            field_index = source.fields().indexFromName('Area_ha')
            if caps & QgsVectorDataProvider.ChangeAttributeValues:
                for feature in source.getFeatures():
                    attr = {field_index : round(feature.geometry().area()/10000, 2)}
                    source.dataProvider().changeAttributeValues({feature.id() : attr})
            source.updateFields()

            field_index_list = []            
            field_name = "Perimetro"
            if source.fields().indexFromName(field_name)>0:
                field_index_list.append(source.fields().indexFromName(field_name))            
            field_name = "PERIMETRO"
            if source.fields().indexFromName(field_name)>0:
                field_index_list.append(source.fields().indexFromName(field_name))            
            field_name = "perimetro"
            if source.fields().indexFromName(field_name)>0:    
                field_index_list.append(source.fields().indexFromName(field_name))
            
            source.dataProvider().deleteAttributes(field_index_list)
            source.updateFields()
            pr = source.dataProvider()
            attr = pr.addAttributes([QgsField('Perimetro', QVariant.Double, 'double', 20, 2)])
            source.updateFields()
            caps = pr.capabilities()
            field_index1 = source.fields().indexFromName('Perimetro')
            if caps & QgsVectorDataProvider.ChangeAttributeValues:
                for feature in source.getFeatures():
                    attr = {field_index1 : round(feature.geometry().length(), 2)}
                    source.dataProvider().changeAttributeValues({feature.id() : attr})
            source.updateFields()
        
        elif geom.type() == QgsWkbTypes.LineGeometry:
            
            field_index_list = []            
            field_name = "Comprt"
            if source.fields().indexFromName(field_name)>0:
                field_index_list.append(source.fields().indexFromName(field_name))            
            field_name = "COMPRT"
            if source.fields().indexFromName(field_name)>0:
                field_index_list.append(source.fields().indexFromName(field_name))            
            field_name = "comprt"
            if source.fields().indexFromName(field_name)>0:    
                field_index_list.append(source.fields().indexFromName(field_name))

            source.dataProvider().deleteAttributes(field_index_list)
            source.updateFields()
            pr = source.dataProvider()
            attr = pr.addAttributes([QgsField('Comprt', QVariant.Double, 'double', 20, 2)])
            source.updateFields()
            caps = pr.capabilities()
            field_index = source.fields().indexFromName('Comprt')
            if caps & QgsVectorDataProvider.ChangeAttributeValues:
                for feature in source.getFeatures():
                    attr = {field_index : round(feature.geometry().length(), 2)}
                    source.dataProvider().changeAttributeValues({feature.id() : attr})
            source.updateFields()

        return {self.OUTPUT: source} 